package org.apache.geronimo;

import java.io.IOException;
import java.net.URL;
import java.util.Date;
import java.util.Random;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;

import org.apache.ahc.AsyncHttpClient;
import org.apache.ahc.codec.HttpRequestMessage;
import org.apache.catalina.CometEvent;
import org.apache.catalina.CometProcessor;

/**
 * Servlet implementation class for Servlet: AsyncServlet
 * 
 * @web.servlet name="AsyncServlet" display-name="AsyncServlet"
 * 
 * @web.servlet-mapping url-pattern="/async"
 * 
 */
public class AsyncServlet extends HttpServlet implements CometProcessor {
	@Override
	public void destroy() {
		// TODO Auto-generated method stub
		super.destroy();
		ahc.destroyAll();
	}

	static final long serialVersionUID = 1L;

	private String remoteUrl;
	private static AsyncHttpClient ahc;

	public void init() throws ServletException {
		remoteUrl = getServletConfig().getInitParameter("remoteUrl");

		ExecutorService threadPool = Executors.newFixedThreadPool(8);
    ahc = new AsyncHttpClient(threadPool);
	}

	/**
	 * Process the given Comet event.
	 * 
	 * @param event
	 *            The Comet event that will be processed
	 * @throws IOException
	 * @throws ServletException
	 */
	public void event(CometEvent event) throws IOException, ServletException {
		HttpServletRequest request = event.getHttpServletRequest();

		String cometId = request.getSession(true).getId();
		System.out.println("");
		System.out.println("Session id: " + cometId + " "
				+ event.getEventType().toString());

		if (event.getEventType() == CometEvent.EventType.BEGIN) {
			event.getHttpServletRequest().setAttribute(
					"org.apache.tomcat.comet.timeout", new Integer(100 * 1000));
			event.setTimeout(100 * 1000);

			if (request.getParameter("remoteUrl") != null) {
				remoteUrl = request.getParameter("remoteUrl");        
			}

			String remoteUri = request.getParameter("remoteUri");
			if (remoteUri == null) {
				remoteUri = "/remoteApp/page";
			}
			
			URL url_connect = new URL(remoteUrl + remoteUri);			
			Callback callback = new Callback(event.getHttpServletResponse(), cometId);
			//Callback callback = new Callback(event, cometId);
			HttpRequestMessage msgRequest = new HttpRequestMessage(url_connect, callback);

			// If the file being served has other URIs in it, like say images,
			// subsequent calls are made to the remote app to retrieve those.
			// We don't want to set new delay times and status codes then.
			if (remoteUri.endsWith("page")) {
				Integer[] params = generateRandomParams();
				msgRequest.setParameter("sessionId", cometId);
				msgRequest.setParameter("delay", params[0].toString());
				msgRequest.setParameter("code", params[1].toString());
				msgRequest.setTimeOut(1500);

				System.out.println("sessionId:" + cometId + "; delay:"
						+ params[0] + "; code:" + params[1]);
			}
      else {
				System.out.println("msg request is " + msgRequest.getPath());
			}

			msgRequest.setRequestMethod(HttpRequestMessage.REQUEST_GET);			
			
			try {
				callback.setTime(new Date().getTime());
				ahc.sendRequest(msgRequest);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
    else if (event.getEventType() == CometEvent.EventType.ERROR) {
			event.close();
		}
    else if (event.getEventType() == CometEvent.EventType.END) {
			event.close();
		}
	}

	/**
	 * @return
	 */
	protected Integer[] generateRandomParams() {
		Integer[] params = new Integer[2];
		final int[] codes = new int[] { 200, 302, 403, 404, 500 };

		Random generator = new Random();

		// generate a random delay between 10ms and 250 ms
		int delay = generator.nextInt(2000) + 10;
		params[0] = new Integer(delay);

		// generate randomly one of http status codes.
		int index = generator.nextInt(codes.length);
		params[1] = new Integer(codes[index]);

		return params;
	}

}